const COPYCOL_LINK_ADDRESS = 1;

// interfaces for the different html elements
const nsIAnchorElement   = Components.interfaces.nsIDOMHTMLAnchorElement
const nsIImageElement    = Components.interfaces.nsIDOMHTMLImageElement
const nsIAreaElement     = Components.interfaces.nsIDOMHTMLAreaElement
const nsILinkElement     = Components.interfaces.nsIDOMHTMLLinkElement
const nsIInputElement    = Components.interfaces.nsIDOMHTMLInputElement
const nsIFormElement     = Components.interfaces.nsIDOMHTMLFormElement
const nsIAppletElement   = Components.interfaces.nsIDOMHTMLAppletElement
const nsIObjectElement   = Components.interfaces.nsIDOMHTMLObjectElement
const nsIEmbedElement    = Components.interfaces.nsIDOMHTMLEmbedElement
const nsIButtonElement   = Components.interfaces.nsIDOMHTMLButtonElement
const nsISelectElement   = Components.interfaces.nsIDOMHTMLSelectElement
const nsITextareaElement = Components.interfaces.nsIDOMHTMLTextAreaElement
const nsIQuoteElement    = Components.interfaces.nsIDOMHTMLQuoteElement
const nsIModElement      = Components.interfaces.nsIDOMHTMLModElement

var pageinfo_link_linkView = null;

//
// load
//
function pageinfo_link_load(){
	theBundle = document.getElementById("pageinfo_link_bundle");
	gStrings.linkAnchor = theBundle.getString("linkAnchor");
	gStrings.linkArea = theBundle.getString("linkArea");
	gStrings.linkSubmit = theBundle.getString("linkSubmit");
	gStrings.linkSubmission = theBundle.getString("linkSubmission");
	gStrings.linkRel = theBundle.getString("linkRel");
	gStrings.linkStylesheet = theBundle.getString("linkStylesheet");
	gStrings.linkRev = theBundle.getString("linkRev");
	gStrings.linkX = theBundle.getString("linkX");
	gStrings.linkQuote = theBundle.getString("linkQuote");
	gStrings.linkMod = theBundle.getString("linkMod");

	// for Firefox 3.0, 3.5
	pageinfo_link_linkView = new pageInfoTreeView(
		["link-name","link-address","link-type","link-target","link-accesskey"],
		COPYCOL_LINK_ADDRESS);
	// for Firefox 3.6+
	// pageinfo_link_linkView = new pageInfoTreeView(COPYCOL_LINK_ADDRESS);

	// Fix pageinfo_link_linkView.copycol for Firefox 3.6+
	pageinfo_link_linkView.copycol = COPYCOL_LINK_ADDRESS;

	var linkTree = document.getElementById("linktree");
	linkTree.treeBoxObject.view = pageinfo_link_linkView;
}
// onLoadRegistry is defined in pageInfo.js.
onLoadRegistry.push(pageinfo_link_load);

//
// processElement
//
function pageinfo_link_processElement(elem){
try {
	var linkView = pageinfo_link_linkView;

	// one swi^H^H^Hif-else to rule them all
	if (elem instanceof nsIAnchorElement)
		linkView.addRow([getValueText(elem), elem.href, gStrings.linkAnchor, elem.target, elem.accessKey]);
	else if (elem instanceof nsIAreaElement)
		linkView.addRow([elem.alt, elem.href, gStrings.linkArea, elem.target]);
	else if (elem instanceof nsIQuoteElement && elem.cite)
		linkView.addRow([getValueText(elem), elem.cite, gStrings.linkQuote]);
	// else if (elem instanceof nsIModElement && elem.cite)
	// 	linkView.addRow([getValueText(elem), elem.cite, gStrings.linkMod]);
	else if (elem instanceof nsILinkElement) {
		if (elem.rel){
			var rel = elem.rel;
			if (/\bicon\b/i.test(rel)) {
			} else if (/\bstylesheet\b/i.test(rel)) {
				linkView.addRow([elem.rel, elem.href, gStrings.linkStylesheet, elem.target]);
			} else {
				linkView.addRow([elem.rel, elem.href, gStrings.linkRel, elem.target]);
			}
		}else {
			linkView.addRow([elem.rev, elem.href, gStrings.linkRev, elem.target]);
		}
	}else if (elem instanceof nsIInputElement || elem instanceof nsIButtonElement) {
		switch (elem.type.toLowerCase())
		{
			case "image":
			// Fall through, <input type="image"> submits
			case "submit":
				if ("form" in elem && elem.form)
				{
					linkView.addRow([elem.value || getValueText(elem) || gStrings.linkSubmit, elem.form.action, gStrings.linkSubmission, elem.form.target]);
				}
				else
					linkView.addRow([elem.value || getValueText(elem) || gStrings.linkSubmit, '', gStrings.linkSubmission, '']);
		}
	}
	else if (elem.hasAttributeNS(XLinkNS, "href")) {
		var ioService = Components.classes["@mozilla.org/network/io-service;1"]
			.getService(Components.interfaces.nsIIOService);
		var href = elem.getAttributeNS(XLinkNS, "href");
		try {
			var baseURI = ioService.newURI(elem.baseURI, elem.ownerDocument.characterSet, null);
			var uri = ioService.newURI(href, elem.ownerDocument.characterSet, baseURI).spec;
		} catch (e) {}
		linkView.addRow([getValueText(elem), uri, gStrings.linkX, ""]);
	}
}catch(e) {
  gWindow.alert("error: " + e);
}
}
// onProcessElement is defined in pageInfo.js.
onProcessElement.push(pageinfo_link_processElement);

//
// doubleclick
//
function pageinfo_link_doubleclick(event) {
	if (event.originalTarget.localName != "treechildren"){ return; }

	var tree = event.target;
	if (!("treeBoxObject" in tree)){ tree = tree.parentNode; }

	var row = {};
	var col = {};
	var elt = {};
	tree.treeBoxObject.getCellAt(event.clientX, event.clientY, row, col, elt);

	if (row.value == -1){ return; }

	var column = tree.columns.getNamedColumn('link-address');
	var url = tree.treeBoxObject.view.getCellText(row.value, column);

	if ("gBrowser" in window.opener){
		window.opener.gBrowser.addTab(url);
	}
}


